;;;
;;;    By Dominic Panholzer
;;;
;;;    ACETLAYR.LSP
;;;    Copyright  1999 by Autodesk, Inc.
;;;
;;;    Your use of this software is governed by the terms and conditions of the
;;;    License Agreement you accepted prior to installation of this software.
;;;    Please note that pursuant to the License Agreement for this software,
;;;    "[c]opying of this computer program or its documentation except as
;;;    permitted by this License is copyright infringement under the laws of
;;;    your country.  If you copy this computer program without permission of
;;;    Autodesk, you are violating the law."
;;;
;;;    AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
;;;    AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
;;;    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
;;;    DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
;;;    UNINTERRUPTED OR ERROR FREE.
;;;
;;;    Use, duplication, or disclosure by the U.S. Government is subject to
;;;    restrictions set forth in FAR 52.227-19 (Commercial Computer
;;;    Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
;;;    (Rights in Technical Data and Computer Software), as applicable.
;;;
;;;  ----------------------------------------------------------------
;;;
;;;
;;;     This file contains a library of layer based routines. See individual
;;;     routines for descriptions.
;;;
;;;  External Functions:
;;;
;;;     init_bonus_error         --> ACETUTIL.FAS   Intializes bonus error routine
;;;     restore_old_error      --> ACETUTIL.FAS   Restores old error routine
;;;     ACET-STR-FORMAT         --> ACETUTIL.ARX   Alternate to strcat
;;;

; -------------------- ISOLATE LAYER FUNCTION --------------------
; Isolates selected object's layer by turning all other layers off
; ----------------------------------------------------------------

(defun init_bonus_error ( lst / ss undo_init)
 
  ;;;;;;;local function;;;;;;;;;;;;;;;;;;;;
  (defun undo_init ( / undo_ctl)
   (b_set_sysvars (list "cmdecho" 0))
   (setq undo_ctl (getvar "undoctl")) 
   (if (equal 0 (getvar "UNDOCTL")) ;Make sure undo is fully enabled.
       (command "_.undo" "_all")
   )
   (if (or (not (equal 1 (logand 1 (getvar "UNDOCTL"))))  
           (equal 2 (logand 2 (getvar "UNDOCTL")))
       );or
       (command "_.undo" "_control" "_all") 
   )
    
   ;Ensure undo auto is off
   (if (equal 4 (logand 4 (getvar "undoctl")))
       (command "_.undo" "_Auto" "_off")
   )
   
   ;Place an end mark down if needed.
   (while (equal 8 (logand 8 (getvar "undoctl")))
        (command "_.undo" "_end")
   );while         
   (while (not (equal 8 (logand 8 (getvar "undoctl"))))
    (command "_.undo" "_begin")                 
   );while
   (b_restore_sysvars) 
   ;return original value of undoctl
   undo_ctl
  );defun undo_init

    ;;;;;;;;;;;;;begin the work of init_bonus error;;;;;;;;;;;;;
 (setq ss (ssgetfirst))
 (while (not (equal (getvar "cmdnames") "")) 
  (command nil)
 );while rk added 6:45 PM 8/10/97
 (if (not bonus_alive)
     (setq bonus_alive 0)
 );if
 (setq bonus_alive (1+ bonus_alive))
 
 (if (and (> bonus_alive 1)                              ;do some double checking to make sure 
          (or (not (equal 'LIST (type *error*)))         ;our error handler is still active.
              (not (equal "bonus_error" (cadr *error*))) ;for nested this call.
          );or
     );and
     (progn
      (princ "\nNested Error trapping is being used incorrectly.")
      (princ "\nResetting the nested index to 1.")
      (setq     *error* bonus_error
            bonus_alive 0
      );setq
      (restore_old_error);quietly restore undo status
      (setq bonus_alive 1)
     );progn then things need to be re-adjusted.
 );if
 (if (<= bonus_alive 0)   
     (progn 
      (setq bonus_alive 0);undo settings will be restored 
                          ;along with setting *error* back to bonus_old_error.
                          ;No call to b_restore_sysvars will be made.
                          ;If it is decided, this thing should do variable clean 
                          ;up also then set bonus_alive to 1 before calling
                          ;restore_old_error
      (restore_old_error);quietly restore bonus_old_error and undo status.
      (setq bonus_alive 1)
     );progn then
 );if
 (if (= bonus_alive 1)
     (progn
      (if (and *error*
               (or (not (equal 'LIST (type *error*)))
                   (not (equal "bonus_error" (cadr *error*)))
               );or 
          );and 
          (setq bonus_old_error *error*);save the *error* only if it 
                                        ;looks like the standard one or is some other 
                                        ;user defined one. Don't want to save it if 
                                        ;it's ours because we already have it.
      );if
      (if (cadr lst)
          (setq bonus_undoctl (undo_init)) 
          (setq bonus_undoctl nil)
      );if
    );progn then this is a top level call, or in other words, the first time through.
 );if
 (b_set_sysvars (car lst))
 (if (= bonus_alive 1)
     (progn
      (setq *error* bonus_error);setq
      (if (caddr lst)
          (setq *error* (append (reverse (cdr (reverse *error*))) 
                                (list (caddr lst)
                                      (last *error*)
                                );list
                        );append
          );setq ;then add additional routine name to the error function.
      );if
     );progn
     (progn
      (if (and (> bonus_alive 1)
               (or (not (equal 'LIST (type *error*)))
                   (not (equal "bonus_error" (cadr *error*)))
               );or
          );and
          (setq *error* bonus_error);setq
      );if
     );progn else double check to make sure the bonus_error is in effect.
 );if
 (if (and ss
          (equal 1 (logand 1 (getvar "pickfirst")))
     );and
     (sssetfirst (car ss) (cadr ss))
 );if
);defun init_bonus_error

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bonus_error ( msg / )

"bonus_error"

(setq bonus_alive -1)
(print msg)

;;Get out of any active command.
(while (not (equal (getvar "cmdnames") ""))
 (command nil)
)

;If undo global variable flag is set then use undo as a cleanup helper.
(if bonus_undoctl
    (progn
     (setvar "cmdecho" 0)

     (while (not (wcmatch (getvar "cmdnames") "*UNDO*"))
            (command "_.undo")
     );while
     (command "_end")  ;The routine that just failed created an undo 
                       ;begin mark, so we need to close it off with 
                       ;and "end" mark.

     (command "_.undo" "1")   ;now back up to the begining.
     (while (not (equal (getvar "cmdnames") "")) 
      (command nil)
     );while

    );progn
);if

(b_restore_sysvars)
(b_restore_undo)

;Restore original error handler
(if bonus_old_error
    (setq *error* bonus_old_error)
);if

(setq bonus_alive 0)

(princ)
);defun bonus_error

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Restore_old_error
;This function should be the last thing called in a lisp 
;defined command. It does a (princ) at the end for a quiet 
;finish.
(defun restore_old_error ( / )


(setq bonus_alive (- bonus_alive 1))
(if (>= bonus_alive 0)
    (b_restore_sysvars)
    (setq bonus_varlist nil)
);if
(if (<= bonus_alive 0)
    (progn
     (while (not (equal (getvar "cmdnames") "")) 
      (command nil)
     );while rk 
     (b_restore_undo)
     (if bonus_old_error
         (setq *error* bonus_old_error);put the old error routine back.
     );if
    );progn then
);if

(princ)
);defun restore_old_error



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun b_restore_undo ()

(if bonus_undoctl
    (progn
      (b_set_sysvars (list "cmdecho" 0))

      (while (equal 8 (logand 8 (getvar "undoctl")))
         (command "_.undo" "_end")
      );while

      (if (not (equal bonus_undoctl (getvar "undoctl")))
          (progn
           (cond 
            ((equal 0 bonus_undoctl) 
             (command "_.undo" "_control" "_none")
            )
            ((equal 2 (logand 2 bonus_undoctl))
             (command "_.undo" "_control" "_one")
            )	
           );;cond 
           (if (equal 4 (logand 4 bonus_undoctl))
               (command "_.undo" "_auto" "_on") 
           );if 

         );progn then restore undoctl to the status the user had it set to. 
      );if
      (if (not (equal 2 (logand 2 (getvar "undoctl"))))
          (b_restore_sysvars)
      );if
    );progn then restore undo to it's original setting
);if
(setq bonus_undoctl nil)

);defun b_restore_undo


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This has no error checking. You must
;provide a list of even length in the 
;following form
;( "sysvar1" value
;  "sysvar2" value2
;)
(defun b_set_sysvars (lst / lst2 lst3 a b n)

(setq lst3 (car bonus_varlist));setq

(setq n 0)
(repeat (/ (length lst) 2)
 (setq a (strcase (nth n lst))
       b (nth (+ n 1) lst)
 );setq
 (setq lst2 (append lst2
                    (list (list a (getvar a)))
            );append
 );setq 
 (if (and bonus_varlist 
          (not (assoc a lst3))
     );and
     (setq lst3 (append lst3 
                        (list (list a (getvar a)))
                );append
     );setq 
 );if

 (setvar a b)

(setq n (+ n 2));setq
);repeat
(if bonus_varlist
    (setq bonus_varlist (append (list lst3) 
                                (cdr bonus_varlist)
                                (list lst2) 
                        );append
    );setq
    (setq bonus_varlist (list lst2))
);if
);defun b_set_sysvars

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun b_restore_sysvars ( / lst n a b)

 (if (<= bonus_alive 0)
     (setq           lst (car bonus_varlist)
           bonus_varlist (list lst)
     );setq 
     (setq lst (last bonus_varlist)) 
 );if

 (setq n 0);setq
 (repeat (length lst)
 (setq a (nth n lst)
       b (cadr a)
       a (car a)
 )
 (setvar a b)
 (setq n (+ n 1));setq
 );repeat
 (setq bonus_varlist (reverse (cdr (reverse bonus_varlist))))

);defun b_restore_sysvars




(Defun C:LAYISO (/ SS CNT LAY LAYLST VAL VPMODE)

  (init_bonus_error
    (list
      (list "cmdecho" 0
            "expert"  0
      )
      T     ;flag. True means use undo for error clean up.
    );list
  );init_bonus_error


  (if (and (= 0 (getvar "tilemode"))                          ; if in a paper space
           (/= 1 (getvar "cvport"))                           ; viewport
      )
    (setq VPMODE T)                                           ; set flag for viewport behavior
  )

  (if (not (setq SS (ssget "_i")))
    (progn
      (prompt "\nObjekt(e) auf zu isolierenden Layern whlen: ")
      (setq SS (ssget))
    )
  )


  (if SS
    (progn

      (setq CNT 0)

      (while (and ss
                  (> (sslength ss) 0)
                  (setq LAY (ssname SS CNT))
             );and
        (setq LAY (cdr (assoc 8 (entget LAY))))
        (if (not (member LAY LAYLST))
          (setq LAYLST (cons LAY LAYLST))
        )
        (setq CNT (1+ CNT))
      )

      (cond
        (VPMODE
          (command "_.vplayer" "_freeze" "*" "_Current")
          (foreach VAL LAYLST (command "_Thaw" VAL "_Current"))
          (command "")
          (if (= (length LAYLST) 1)
            (prompt (acet-str-format "\nLayer %1 wurde in diesem Ansichtsfenster isoliert." (car LAYLST)))
            (prompt (acet-str-format "\n%1 Layer wurden in diesem Ansichtsfenster isoliert." (itoa (length LAYLST))))
          )
        )
        (T
          (if (member (getvar "CLAYER") LAYLST)
            (setq LAY (getvar "CLAYER"))
            (setvar "CLAYER" (setq LAY (last LAYLST)))
          )

          (command "_.-LAYER" "_OFF" "*" "_Y")
          (foreach VAL LAYLST (command "_ON" VAL))
          (command "")

          (if (= (length LAYLST) 1)
            (prompt (acet-str-format "\nLayer %1 wurde isoliert." (car LAYLST)))
            (prompt (acet-str-format "\n%1 Layer wurden isoliert. %2 ist der aktuelle Layer."
                                      (itoa (length LAYLST)) LAY)
            )
          )
        )
      )
    )
  )

  (restore_old_error)

  (princ)
)

; -------------------- LAYER FREEZE FUNCTION ---------------------
; Freezes selected object's layer
; ----------------------------------------------------------------

(defun C:LAYFRZ ()
; --------------------- Error initialization ---------------------

  (init_bonus_error
    (list
      (list "cmdecho" 0
            "expert"  0
      )

      nil     ;flag. True means use undo for error clean up.
    );list
  );init_bonus_error

  (layproc "frz")

  (restore_old_error)
  (princ)
)

; ---------------------- LAYER OFF FUNCTION ----------------------
; Turns selected object's layer off
; ----------------------------------------------------------------

(defun C:LAYOFF ()
; --------------------- Error initialization ---------------------

  (init_bonus_error
    (list
      (list "cmdecho" 0
            "expert"  0
      )

      nil     ;flag. True means use undo for error clean up.
    );list
  );init_bonus_error

  (layproc "off")

  (restore_old_error)
  (princ)
)

; ------------- LAYER PROCESSOR FOR LAYOFF & LAYFRZ --------------
; Main program body for LAYOFF and LAYFRZ. Provides user with
; options for handling nested entities.
; ----------------------------------------------------------------

(defun LAYPROC ( TASK / NOEXIT OPT BLKLST CNT VPMODE EN PMT ANS LAY NEST BLKLST VPSS)



; -------------------- Variable initialization -------------------

  (setq NOEXIT T)

  (setq OPT (getenv (strcat "ACET_Lay" TASK)))    ; get default option setting
  (if (not (or (null OPT) (= OPT ""))) (setq OPT (atoi OPT)))

  (setq CNT 0)                                                ; cycle counter

  (if (and (= 0 (getvar "tilemode"))                          ; if in a paper space
           (/= 1 (getvar "cvport"))                           ; viewport
      )
    (setq VPMODE T)                                           ; set flag for freeze behavior
  )


; ----------------------- Selection Prompt -----------------------

  (while NOEXIT

    (setvar "errno" 7)
    (while (= (getvar "errno") 7)
      (setvar "errno" 0)
      (initget "Optionen Zurck _Options Undo ")
      (cond
        ((= TASK "off")
          (setq EN (nentsel "\nEin Objekt auf dem auszuschaltenden Layer whlen oder [Optionen/Zurck]: "))
        )
        ((= TASK "frz")
          (setq EN (nentsel "\nEin Objekt auf dem zu frierenden Layer whlen oder [Optionen/Zurck]: "))
        )
        ((= TASK "vpi")
          (setq EN (nentsel "\nEin Objekt auf dem im Ansichtsfenster zu isolierenden Layer whlen oder [Optionen/Zurck]: "))
        )
      )
      (if (= (getvar "errno") 7)
        (prompt "\nNichts gewhlt.")
      )
    )

; ---------------------- Options  Selected -----------------------

    (cond
      ((= EN "Options")
        (initget "Nicht Blcke Objekte _No Block Entity ")
        (cond
          ((= OPT 1)
            (setq PMT "\nEine Option eingeben [Blcke verschachteln/Objekte verschachteln/]<Nicht verschachteln>: ")
          )
          ((= OPT 2)
            (setq PMT "\nEine Option eingeben [Blcke verschachteln/Nicht verschachteln/]<Objekte verschachteln>: ")
          )
          (T
            (setq PMT "\nEine Option eingeben [Objekte verschachteln/Nicht verschachteln/]<Blcke verschachteln>: ")
          )
        )
        (setq ANS (getkword PMT))

        (cond
          ((null ANS)
            (if (or (null OPT) (= OPT ""))
              (progn
                (setq OPT 3)
                (setenv (strcat "ACET_Lay" TASK) "3")
              )
            )
          )
          ((= ANS "No")
            (setq OPT 1)
            (setenv (strcat "ACET_Lay" TASK) "1")
          )
          ((= ANS "Entity")
            (setq OPT 2)
            (setenv (strcat "ACET_Lay" TASK) "2")
          )
          (T
            (setq OPT 3)
            (setenv (strcat "ACET_Lay" TASK) "3")
          )
        )
      )


; ---------------------- Undo selected ---------------------------

      ((= EN "Undo")
        (if (> CNT 0)
          (progn
            (command "_.u")
            (setq CNT (1- CNT))
          )
          (prompt "\nEs wurde bereits alles rckgngig gemacht.")
        )
      )

; ------------------------- Find Layer ---------------------------

    (EN

        (setq BLKLST (last EN))
        (setq NEST (length BLKLST))

        (cond

      ; If the entity is not nested or if the option for entity
      ; level nesting is selected.

          ((or (= OPT 2) (< (length EN) 3))
            (setq LAY (entget (car EN)))
          )

      ; If no nesting is desired

          ((= OPT 1)
            (setq LAY (entget (car (reverse BLKLST))))
          )

      ; All other cases (default)

          (T
            (setq BLKLST (reverse BLKLST))

            (while (and                         ; strip out xrefs
                ( > (length BLKLST) 0)
                (assoc 1 (tblsearch "BLOCK" (cdr (assoc 2 (entget (car BLKLST))))))
                   );and
              (setq BLKLST (cdr BLKLST))
            )
            (if ( > (length BLKLST) 0)          ; if there is a block present
              (setq LAY (entget (car BLKLST)))  ; use block layer
              (setq LAY (entget (car EN)))      ; else use layer of nensel
            )
          )
        )

; ------------------------ Process Layer -------------------------

        (setq LAY (cdr (assoc 8 LAY)))

        (if (= LAY (getvar "CLAYER"))
          (cond
            ((= TASK "off")
              (initget "Ja Nein _Yes No")
              (setq ANS (getkword (acet-str-format "\nLayer %1 (AKTUELLER Layer) wirklich ausschalten? [Ja/Nein] <Nein>: " LAY)))
              (setq ANS (if (null ANS) "No" ANS))
              (if (= ANS "No")
                (setq LAY nil)
              )
            )
            ((and (= TASK "frz") (not VPMODE))
              (prompt (acet-str-format "\nLayer %1 ist der AKTUELLE Layer und kann nicht gefroren werden." LAY))
              (setq LAY nil)
            )
          )
          (setq ANS nil)
        )

        (if LAY
          (cond
            ((= TASK "off")
              (if ANS
                (command "_.-LAYER" "_OFF" LAY "_Yes" "")
                (command "_.-LAYER" "_OFF" LAY "")
              )
              (prompt (acet-str-format "\nLayer %1 wurde ausgeschaltet." LAY))
              (setq CNT (1+ CNT))
            )
            ((and (= TASK "frz") VPMODE)
              (command "_.VPLAYER" "_FREEZE" LAY "_current" "")
              (prompt (acet-str-format "\nLayer %1 wurde in diesem Ansichtsfenster gefroren." LAY))
              (setq CNT (1+ CNT))
            )
            ((= TASK "frz")
              (command "_.-LAYER" "_FREEZE" LAY "")
              (prompt (acet-str-format "\nLayer %1 wurde gefroren."  LAY ))
              (setq CNT (1+ CNT))
            )
            ((= TASK "vpi")
              (setq VPSS (ssget "_x" (list '(-4 . "<AND")
                                             '(0 . "VIEWPORT")                 ; get all viewports
                                             '(-4 . "<NOT")
                                                (cons 69 (getvar "cvport"))    ; except the current
                                             '(-4 . "NOT>")
                                             '(-4 . "<NOT")
                                                '(69 . 1)                      ; and the paperspace viewport (1)
                                             '(-4 . "NOT>")
                                           '(-4 . "AND>")
                                     )
                         )
              )
              (command "_.VPLAYER" "_FREEZE" LAY "_select" VPSS "" "")
              (prompt (acet-str-format "\nLayer %1 wurde in allen auer dem aktuellen Ansichtsfenster gefroren."  LAY ))
              (setq CNT (1+ CNT))
            )
          )
        )
      )

; ---------------------- Nothing  Selected -----------------------

      ((not EN)
        (setq NOEXIT nil)
      )
    )
  )
)

; --------------------- LAYER LOCK FUNCTION ----------------------
; Locks selected object's layer
; ----------------------------------------------------------------

(Defun C:LAYLCK (/ LAY)

  (init_bonus_error
    (list
      (list "cmdecho" 0
            "expert"  0
      )

      T     ;flag. True means use undo for error clean up.
    );list
  );init_bonus_error

  (setq LAY (entsel "\nEin Objekt auf dem zu sperrenden Layer whlen: "))

  (if LAY
    (progn
      (setq LAY (cdr (assoc 8 (entget (car LAY)))))
      (Command "_.-LAYER" "_LOCK" LAY "")
      (prompt (acet-str-format "\nLayer %1 wurde gesperrt."  LAY ))
    )
  )

  (restore_old_error)

  (princ)
)

; -------------------- LAYER UNLOCK FUNCTION ---------------------
; Unlocks selected object's layer
; ----------------------------------------------------------------

(Defun C:LAYULK (/ LAY)

  (init_bonus_error
    (list
      (list "cmdecho" 0
            "expert"  0
      )

      T     ;flag. True means use undo for error clean up.
    );list
  );init_bonus_error

  (setq LAY (entsel "\nEin Objekt auf dem zu entsperrenden Layer whlen: "))

  (if LAY
    (progn
      (setq LAY (cdr (assoc 8 (entget (car LAY)))))
      (Command "_.-LAYER" "_UNLOCK" LAY "")
      (prompt (acet-str-format "\nLayer %1 wurde entsperrt."  LAY ))
    )
  )

  (restore_old_error)

  (princ)
)

; ---------------------- LAYER ON FUNCTION -----------------------
; Turns all layers on
; ----------------------------------------------------------------

(Defun C:LAYON ()

  (init_bonus_error
    (list
      (list "cmdecho" 0)
      nil     ;flag. True means use undo for error clean up.
    );list
  );init_bonus_error

  (Command "_.-LAYER" "_ON" "*" "")
  (prompt "\nAlle Layer wurden eingeschaltet.")

  (restore_old_error)

  (princ)
)


; --------------------- LAYER THAW FUNCTION ----------------------
; Thaws all layers
; ----------------------------------------------------------------

(Defun C:LAYTHW ()

  (init_bonus_error
    (list
      (list "cmdecho" 0)
      nil     ;flag. True means use undo for error clean up.
    );list
  );init_bonus_error

  (Command "_.-LAYER" "_THAW" "*" "")
  (prompt "\nAlle Layer wurden getaut.")

  (restore_old_error)

  (princ)
)


; --------------------- LAYER MATCH FUNCTION ---------------------
; Changes the layer of selected object(s) to the layer of a
; selected destination object.
; ----------------------------------------------------------------

(Defun C:LAYMCH (/ SS CNT LOOP LAY ANS)

  (init_bonus_error
    (list
      (list "cmdecho" 0)
      T     ;flag. True means use undo for error clean up.
    );list
  );init_bonus_error


  (if (not (setq SS (ssget "_i")))
    (progn
      (prompt "\nZu ndernde Objekte whlen: ")
      (setq SS (ssget))
    )
  )

  (if SS
    (progn
      (setq CNT (sslength SS))
      (princ (acet-str-format "\n%1 gefunden."  (itoa CNT)))       ; Report number of items found

      (command "_.move" SS "")                         ; filter out objects on locked layers

      (if (> (getvar "cmdactive") 0)                   ; if there are still objects left
        (progn
          (command "0,0" "0,0")
          (setq SS  (ssget "_p")
                CNT (- CNT (sslength SS))              ; count them
          )
        )
        (setq SS nil)                                  ; else abort operation
      )

      (if (> CNT 0)                                    ; if items where filtered out
        (if (= CNT 1)
          (princ (acet-str-format "\n%1 befand sich auf einem gesperrten Layer."  (itoa CNT) ))   ; report it.
          (princ (acet-str-format "\n%1 befanden sich auf einem gesperrten Layer."  (itoa CNT) ))
        )
      )
    )
  )


  (if SS
    (progn
      (initget "Eingeben _Type-it ")
      (setq LAY  (entsel "\nObjekt auf Ziellayer auswhlen oder [Eingeben]: ")
            LOOP T
      )

      (while LOOP
        (cond
          ((not LAY)
            (prompt "\nNichts gewhlt.")
            (initget "Ja Nein _Yes No")
            (setq ANS (getkword "\nAktuellen Layer verwenden? [Ja/Nein] <Ja> "))
            (setq ANS (if (null ANS) "Yes" ANS))
            (if (= ANS "Yes")
              (setq LAY  (getvar "clayer")
                    LOOP nil
              )
            )
          )
          ((listp LAY)
            (setq LOOP nil)
          )
          ((= LAY "Type-it")
            (setq LAY (getstring "\nLayernamen eingeben: "))
            (cond
              ((tblobjname "LAYER" LAY)
                (setq LOOP nil)
              )
              ((/= LAY "")
                (initget "Ja Nein _Yes No ")
                (setq ANS (getkword "\nLayer nicht vorhanden. Soll er erstellt werden? [Ja/Nein] <Ja>: "))
                (setq ANS (if (null ANS) "Yes" ANS)) ;setq
                (if (= ANS "Yes")
                  (if
                    (entmake (list
                              '(0 . "LAYER")
                              '(100 . "AcDbSymbolTableRecord")
                              '(100 . "AcDbLayerTableRecord")
                              '(6 . "CONTINUOUS")
                              '(62 . 7)
                              '(70 . 0)
                               (cons 2 LAY)
                             )
                    )
                    (setq LOOP nil)
                    (prompt "\nUngltiger Layername.")
                  )
                )
              )
            )
          )
        )
        (if LOOP
          (progn
            (initget "Eingeben _Type-it")
            (setq LAY (entsel "\nObjekt auf Ziellayer auswhlen oder [Eingeben]: "))
          )
        )
      ); while LOOP


      (if (listp LAY)
        (setq LAY (cdr (assoc 8 (entget (car LAY)))))
      )

      (command "_.chprop" SS "" "_la" LAY "")

      (if (= (sslength SS) 1)
        (prompt (acet-str-format "\nEin Objekt wurde auf Layer %1 verschoben" LAY ))
        (prompt (acet-str-format "\n%1 Objekte wurden auf Layer %2 verschoben" (itoa (sslength SS)) LAY ))
      )
      (if (= LAY (getvar "clayer"))
        (prompt " (aktueller Layer).")
        (prompt ".")
      )
    )
  )

  (restore_old_error)

  (princ)
)

; --------------- CHANGE TO CURRENT LAYER FUNCTION ---------------
; Changes the layer of selected object(s) to the current layer
; ----------------------------------------------------------------

(Defun C:LAYCUR (/ SS CNT LAY)

  (init_bonus_error
    (list
      (list "cmdecho" 0)
      T     ;flag. True means use undo for error clean up.
    );list
  );init_bonus_error


  (if (not (setq SS (ssget "_i")))
    (progn
      (prompt "\nObjekte auswhlen, die auf den aktuellen Layer verschoben werden sollen: ")
      (setq SS (ssget))
    )
  )

  (if SS
    (progn
      (setq CNT (sslength SS))
      (princ (acet-str-format "\n%1 gefunden."  (itoa CNT) ))       ; Report number of items found

      (command "_.move" SS "")                         ; filter out objects on locked layers

      (if (> (getvar "cmdactive") 0)                   ; if there are still objects left
        (progn
          (command "0,0" "0,0")
          (setq SS  (ssget "_p")
                CNT (- CNT (sslength SS))              ; count them
          )
        )
        (setq SS nil)                                  ; else abort operation
      )

      (if (> CNT 0)                                    ; if items where filtered out
        (if (= CNT 1)
          (princ (acet-str-format "\n%1 befand sich auf einem gesperrten Layer."  (itoa CNT) ))   ; report it.
          (princ (acet-str-format "\n%1 befanden sich auf einem gesperrten Layer."  (itoa CNT) ))
        )
      )
    )
  )

  (if SS
    (progn
      (setq LAY (getvar "CLAYER"))

      (command "_.chprop" SS "" "_la" LAY "")

      (if (= (sslength SS) 1)
        (prompt (acet-str-format "\nEin Objekt wurde auf Layer %1 verschoben (aktueller Layer)."  LAY ))
        (prompt (acet-str-format "\n%1 Objekte wurden auf Layer %2 verschoben (aktueller Layer)." (itoa (sslength SS)) LAY))
      )
    )
  )

  (restore_old_error)

  (princ)
)
(princ)